<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 10/22/2009 *
 * ===================                          Last change: 10/22/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : cachesystem.class.php                            *
 * -------------------------------------------------------------------- *
 * Short description : CacheSystem class                                *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : CacheSystem-Klasse                               *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Caching class
class CacheSystem {
	// Status code
	public $statusCode = 'init';

	// Full-qualified filename
	public $fqfn = '';

	// Resource to cache file
	public $pointer = FALSE;

	// Data array from cache
	public $data = array();

	// Version data from cache
	public $version = array();

	// Cache name
	public $name = '';
	public $rebuilt = array();

	// File extension
	public $extension = '';
	public $status = array();
	public $readable = array();
	public $fullPath = '';

	// Constructor
	public function __construct () {
		// Set extension
		$this->extension = getCacheExtension();

		// Construct full path
		$this->fullPath = getPath() . getCachePath();

		// Failed is the default
		$this->setStatusCode('failed');

		// Check if path exists
		if (isDirectory($this->fullPath)) {
			// Is there a .htaccess file?
			if (isFileReadable($this->fullPath . '.htaccess')) {
				// All done!
				$this->setStatusCode('done');
			} else {
				// Stop! Set a .htaccess file first
				$this->setStatusCode('htaccess');
			}
		} // END - if
	}

	// Checks validity of cache file and if content is given
	function loadCacheFile ($cacheName) {
		// Remember cache file
		$this->name = $cacheName;

		// Construct FQFN (full qualified file name)
		$this->fqfn = $this->fullPath . $cacheName . $this->extension;

		// Check if file exists and if version matches
		if (!isset($this->status[$cacheName])) {
			// Pre-fetch cache here if found
			if ($this->isCacheReadable()) $this->getArrayFromCache();

			//* DEBUG: */ debugOutput('cacheName='.$cacheName.',isCacheReadable='.intval($this->isCacheReadable()).',is_writeable='.intval(is_writeable($this->fqfn)).',extensionMatches='.intval($this->extensionVersionMatches('cache')));
			$this->status[$cacheName] = ($this->isCacheReadable() && (is_writeable($this->fqfn)) && ($this->extensionVersionMatches('cache')));
		} // END - if
		//* DEBUG: */ debugOutput('cacheName='.$cacheName.',status='.intval($this->status[$cacheName]));

		// Return status
		return $this->status[$cacheName];
	}

	// Initializes the cache file
	function init () {
		// This will destory an existing cache file!
		if ($this->getStatusCode() == 'done') {
			// Reset read status
			$this->resetCacheReadStatus();

			// Create file
			if ($this->isCacheReadable()) changeMode($this->fqfn, 0666);
			$this->pointer = fopen($this->fqfn, 'w') or reportBug(__METHOD__, __LINE__, 'Cannot write to cache ' . $this->fqfn . ' !');

			// Add open PHP tag
			$this->writeLine('<?php');
		} else {
			// Cannot create file
			reportBug(__METHOD__, __LINE__, 'Problems with cache directory detected, getStatusCode()=' . $this->getStatusCode());
		}
	}

	/**
	 * Writes a line to the pointer and adds a \n (new-line) to the end
	 *
	 * @access private
	 */
	function writeLine ($line) {
		// Is the pointer a valid resource?
		if (is_resource($this->pointer)) {
			// Write the line
			fwrite($this->pointer, $line . PHP_EOL);
		} else {
			// Something bad happened
			reportBug(__METHOD__, __LINE__, 'Pointer type is ' . gettype($this->pointer) . ', expected is resource.');
		}
	}

	// Reset the read status
	function resetCacheReadStatus () {
		unset($this->readable[$this->name]);
		unset($GLOBALS['file_readable'][$this->fqfn]);
		unset($this->status[$this->name]);
	}

	// Adds a data row (array) to cache file and global cache array
	function addRow ($data) {
		// Is the pointe rvalid?
		if (is_resource($this->pointer)) {
			// Write every array element to cache file
			foreach ($data as $k => $v) {
				// Write global cache array for immediate access
				if ((substr($k, 0, 4) == 'ext_') && (isset($data['ext_name'])) && (isset($data['ext_id']))) {
					if ($k == 'ext_name') {
						$GLOBALS['cache_array']['extension'][$k][$data['ext_id']] = $v;
					} else {
						$GLOBALS['cache_array']['extension'][$k][$data['ext_name']] = $v;
					}
					if (($k == 'ext_keep') && ($v == 'Y')) {
						$GLOBALS['cache_array']['always_active'][$data['ext_name']] = $v;
					} // END - if
				} elseif ($this->name == 'config') {
					// Configuration
					$GLOBALS['cache_array']['config'][$data['config']][$k] = $v;
				} elseif ($this->name == 'filter') {
					// Filter
					$GLOBALS['cache_array']['filter']['chains'][$data['filter_name']][$data['filter_function']]  = $data['filter_active'];
					$GLOBALS['cache_array']['filter']['counter'][$data['filter_name']][$data['filter_function']] = $data['filter_counter'];
					$GLOBALS['cache_array']['filter']['loaded'][$data['filter_name']][$data['filter_function']]  = TRUE;
				} elseif ($this->name == 'modules') {
					// Modules
					$GLOBALS['cache_array']['modules'][$k][$data['module']] = $v;
				} elseif ($this->name == 'admin') {
					// Admin logins
					if ($k == 'admin_id') {
						$GLOBALS['cache_array']['admin'][$k][$data['login']] = $v;
					} else {
						$GLOBALS['cache_array']['admin'][$k][$data['admin_id']] = $v;
					}
				} elseif ($this->name == 'admin_acls') {
					// Access control lines
					array_push($GLOBALS['cache_array']['admin_acls'][$k][$data['admin_id']], $v);
				} elseif ($this->name == 'refdepths') {
					// Referral levels
					$GLOBALS['cache_array']['refdepths'][$k][$data['id']] = $v;
				} elseif ($this->name == 'refsystem') {
					// Referral system
					$GLOBALS['cache_array']['refsystem'][$k][$data['id']] = $v;
				} elseif ($this->name == 'themes') {
					// Themes
					if ($k == 'theme_path') {
						$GLOBALS['cache_array']['themes'][$k][$data['id']] = $v;
					} else {
						$GLOBALS['cache_array']['themes'][$k][$data['theme_path']] = $v;
					}
				} elseif ($this->name == 'imprint') {
					// Imprint
					$GLOBALS['cache_array']['imprint'][$k][$data['imprint_id']] = $v;
				} elseif ($this->name == 'points_data') {
					// Table 'points_data'
					$GLOBALS['cache_array']['points_data'][$k][$data['id']] = $v;
				} elseif ($this->name == 'earning') {
					// Table 'earning'
					$GLOBALS['cache_array']['earning'][$k][$data['earning_id']] = $v;
				} elseif ($this->name == 'payments') {
					// Table 'payments'
					$GLOBALS['cache_array']['payments'][$k][$data['id']] = $v;
				} elseif (is_array($v)) {
					// Serialize and BASE64-encode the array
					$v = base64_encode(serialize($v));
				} else {
					// Finialize the cache and close it
					$this->finalize();

					// Remove cache
					$this->removeCacheFile(TRUE);

					// Unsupported/unhandled cache detected
					reportBug(__METHOD__, __LINE__, 'Unsupported cache ' . $this->name . ' detected, data=' . print_r($data, TRUE) . ',k=' . $k . ',v=' . $v);
				}

				// Write cache line to file
				$this->writeLine($this->rewriteEntry($k, $v));
			} // END - foreach
		} else {
			// Cannot create file
			reportBug(__METHOD__, __LINE__, 'Problem with cache detected, no resource! pointer[]=' . gettype($this->pointer));
		}
	}

	// Closes cache file with closing PHP tag
	function finalize () {
		// Quit function when no pointer is set
		if (is_resource($this->pointer)) {
			// Add default depency
			$this->storeExtensionVersion('cache');

			// Write footer
			$this->writeLine('?>');

			// Close file add destroy handler
			fclose($this->pointer);

			// Reset readable status
			$this->resetCacheReadStatus();

			// Set rights
			if ($this->isCacheReadable()) changeMode($this->fqfn, 0666);

			// Remove pointer and status
			unset($this->status[$this->name]);
			$this->pointer = FALSE;
			//* DEBUG: */ logDebugMessage(__METHOD__, __LINE__, 'name=' . $this->name . ' - FINALIZED!');
		} // END - if
	}

	// Loads cache file and returns an array of the cached data
	function getArrayFromCache () {
		// Is the cache already loaded?
		if (isset($this->data[$this->name])) {
			// Return it's content!
			return $this->data[$this->name];
		} // END - if

		// Is the cache file there?
		if ($this->isCacheReadable()) {
			// Load cache file
			include($this->fqfn);

			// Is there an array?
			if (isset($this->data[$this->name])) {
				// Cache version found?
				if (isset($this->version[$this->name])) {
					// Check it here if cache matches ext-cache version
					if (!$this->extensionVersionMatches('cache')) {
						// Invalid
						logDebugMessage(__METHOD__, __LINE__, 'Invalid cache data ' . $this->name . ' detected.');
						$this->removeCacheFile();
					} // END - if
				} // END - if

				// Return cache if detected
				if (isset($this->data[$this->name])) {
					return $this->data[$this->name];
				} else {
					// Damaged!
					logDebugMessage(__METHOD__, __LINE__, 'Possible damaged cache ' . $this->name . ' detected.');
					$this->removeCacheFile();
				}
			} // END - if
		} else {
			// Cache file not found or not readable
			reportBug(__METHOD__, __LINE__, '{%message,CACHE_CANNOT_LOAD=' . $this->name . '%}');

			// Try to remove it
			$this->removeCacheFile();
		}

		// Always return an empty array if we have trouble or no data
		return array();
	}

	// Destroy an existing cache file
	function removeCacheFile ($force = FALSE) {
		// Reset read status
		$this->resetCacheReadStatus();

		// Debug message
		//* DEBUG: */ logDebugMessage(__METHOD__, __LINE__, sprintf('%s should be removed.', $this->name));

		// Is the cache file not yet rebuilt?
		if ((!isset($this->rebuilt[$this->name])) && ($this->isCacheReadable())) {
			// Only run in regular output mode
			if ((!isHtmlOutputMode()) && ($force === FALSE)) {
				// Debug message if allowed
				if (isDebugModeEnabled()) {
					// Debug message
					logDebugMessage(__METHOD__, __LINE__, 'Not removing cache ' . $this->name . ' in output_mode=' . getScriptOutputMode());
				} // END - if

				// Abort here
				return;
			} // END - if

			// Close cache
			$this->finalize();

			// Debug-mode enabled?
			if (isDebugModeEnabled()) {
				// Log removal of cache
				logDebugMessage(__METHOD__, __LINE__, 'removing cache: ' . $this->name);
			} // END - if

			// Remove cache file from system
			//* DEBUG: */ reportBug(__METHOD__, __LINE__, 'About to remove ' . basename($this->fqfn) . '!');
			removeFile($this->fqfn);

			// Reset read status
			$this->resetCacheReadStatus();

			// Is the file there?
			if (!$this->isCacheReadable()) {
				// The cache does no longer exist so kill the content
				unset($this->data[$this->name]);
				unset($this->version[$this->name]);
				$this->rebuilt[$this->name] = TRUE;
			} else {
				// Not removed!
				reportBug(__METHOD__, __LINE__, '{%message,CACHE_CANNOT_UNLINK=' . $this->name . '%}');
			}
		} // END - if
	}

	// Unused method:
	function removeEntry ($search, $data, $array) {
		if ($this->status[$this->name] == 'done') {
			// Load cache into dummy array
			$dummy = $this->getArrayFromCache();

			// Search for key in array
			$key = array_search($data, $dummy[$search]);
			if (!empty($key)) {
				// Key (hopefully) found?
				foreach ($array as $a) {
					// So we can remove all elements as requested
					unset($dummy[$a][$key]);
				} // END - foreach

				// Flush array to cache file
				$this->init();

				// Write array out
				$this->writeArray($dummy);

				// Close cache file
				$this->finalize();
			} // END - if
		} else {
			// Cannot write to cache!
			reportBug(__METHOD__, __LINE__, 'Problem with cache detected: Unexpected status ' . $this->status[$this->name]);
		}
	}

	function writeArray ($array) {
		if (is_resource($this->pointer)) {
			foreach ($array as $k => $v) {
				if (is_array($v)) {
					// Multi line(s) found
					$LINE = '';
					foreach ($v as $k2 => $v2) {
						// Put every array element in a row...
						$LINE .= $this->rewriteEntry($k, $v2);
					} // END - foreach
				} else {
					// Single line found
					$LINE = $this->rewriteEntry($k, $v);
				}

				// Write line(s)
				$this->writeLine($LINE);
			} // END - foreach
		} else {
			// Cannot write array!
			reportBug(__METHOD__, __LINE__, 'Problem with cache detected: pointer is not resource! pointer[]=' . gettype($this->pointer));
		}
	}

	// Unused method
	function replaceEntry ($search, $replace, $search_key, $array) {
		if ($this->status[$this->name] == 'done') {
			// Load cache into dummy array
			$dummy = $this->getArrayFromCache();

			// Check if $dummy is valid (prevents some errors)
			if ((is_array($dummy)) && (isset($dummy[$search])) && (is_array($dummy[$search]))) {
				// Search for key in array
				$key_found = array_key_exists($search_key, $dummy[$search]);
				if ($key_found == TRUE) {
					$key = $search_key;
					// Key (hopefully) found?
					foreach ($dummy as $a => $v) {
						// So we can update all entries
						if ($a == $search) {
							// Update now...
							$dummy[$a][$search_key] = $replace;
						} // END - if
					} // END - foreach

					// Flush array to cache file
					$this->init();

					// Write array out
					$this->writeArray($dummy);

					// Close cache file
					$this->finalize();
				} // END - if
			} // END - if
		} else {
			// Cannot write to cache!
			reportBug(__METHOD__, __LINE__, 'Problem with cache detected: Unexpected status ' . $this->status[$this->name]);
		}
	}

	// Writes the version of given extension to the cache file
	function storeExtensionVersion ($ext_name) {
		// Valid cache pointer?
		if (is_resource($this->pointer)) {
			// Write only if extension is installed
			if (isExtensionInstalled($ext_name)) {
				// Get extension version
				$ext_ver = getExtensionVersion($ext_name);

				// Add the extension version to object (DO NOT REMOVE IT! Endless loop...)
				$this->version[$this->name][$ext_name] = $ext_ver;

				// Write cache line to file
				$this->writeLine($this->rewriteEntry($ext_name, $ext_ver, 'version', TRUE));
			} // END - if
			//* DEBUG: */ logDebugMessage(__METHOD__, __LINE__, 'name=' . $this->name . ',ext_name=' . $ext_name . ',ext_ver=' . $ext_ver);
		} else {
			// Cannot create file
			reportBug(__METHOD__, __LINE__, 'Problem with cache detected: pointer is not resource! pointer[]=' . gettype($this->pointer));
		}
	}

	// Checks whether versions from cache and extension matches
	function extensionVersionMatches ($ext_name) {
		// Check cache
		if (!isset($GLOBALS[__METHOD__][$ext_name])) {
			// Does never match by default
			$GLOBALS[__METHOD__][$ext_name] = FALSE;

			// Compare only if installed
			if (isExtensionInstalled($ext_name)) {
				// Get extension version
				$ext_ver = getExtensionVersion($ext_name);

				// Debug messages
				if (isset($this->version[$this->name][$ext_name])) {
					// Does it match?
					$GLOBALS[__METHOD__][$ext_name] = ($this->version[$this->name][$ext_name] == $ext_ver);
				} elseif ($this->isCacheReadable()) {
					// No cache version found
					logDebugMessage(__METHOD__, __LINE__, 'Cache ' . $this->name . ' has missing version entry for extension ext-' . $ext_name . '! Purging cache...');
	
					// Remove the cache file
					$this->removeCacheFile(TRUE);
				}
			} else {
				// Not installed, does always match
				$GLOBALS[__METHOD__][$ext_name] = TRUE;
			}
		} else {
			// Cache entry found, log debug message
			//* DEBUG: */ logDebugMessage(__METHOD__, __LINE__, 'ext_name=' . $ext_name . ', matches=' . intval($GLOBALS[__METHOD__][$ext_name]));
		}

		// Compare both
		return $GLOBALS[__METHOD__][$ext_name];
	}

	// Rewrit the entry so it can be stored in cache file
	// @TODO Add support for more types which break in last else-block
	function rewriteEntry ($key, $value, $prefix = 'data', $single = FALSE) {
		// Default is not single entry
		$extender = '[]';

		// Add only for single array entry?
		if ($single === TRUE) {
			$extender = '';
		} // END - if

		// Init line
		$line = '';

		// String or non-string? ;-)
		if (is_string($value)) {
			// String...
			$line = '$this->' . $prefix . "['" . $this->name . "']['" . $key . "']" . $extender . " = '" . escapeQuotes($value) . "';";
		} elseif (is_null($value)) {
			// Null
			$line = '$this->' . $prefix . "['" . $this->name . "']['" . $key . "']" . $extender . ' = NULL;';
		} elseif (is_bool($value)) {
			// Boolean value
			if ($value === TRUE) {
				// True
				$line = '$this->' . $prefix . "['" . $this->name . "']['" . $key . "']" . $extender . ' = TRUE;';
			} else {
				// False
				$line = '$this->' . $prefix . "['" . $this->name . "']['" . $key . "']" . $extender . ' = FALSE;';
			}
		} elseif (isset($value[0])) {
			// These lines needs fixing
			reportBug(__METHOD__, __LINE__, 'Invalid entry with [0] found. key=' . $key);
		} else {
			// Non-string
			$line = '$this->' . $prefix . "['" . $this->name . "']['" . $key . "']" . $extender . ' = ' . $value . ';';
		}

		// Return line
		return $line;
	}

	// Getter for cache status code
	function getStatusCode () {
		return $this->statusCode;
	}

	// Setter for cache status code
	function setStatusCode ($status) {
		$this->statusCode = $status;
	}

	// Checks whether the current cache file is readable
	function isCacheReadable () {
		// Array entry found?
		if (!isset($this->readable[$this->name])) {
			// Not found, so create it
			$this->readable[$this->name] = isFileReadable($this->fqfn);
		} // END - if

		// Return result
		return $this->readable[$this->name];
	}

	// Cloning not allowed
	function __clone () {
		// Please do not clone this class
		reportBug(__METHOD__, __LINE__, 'Cloning of this class is not allowed.');
	}
} // END - class

// [EOF]
?>
