<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 03/10/2009 *
 * ===================                          Last change: 03/10/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : inc-functions.php                                *
 * -------------------------------------------------------------------- *
 * Short description : Special functions for handling include files     *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Spezielle Funktionen fuer Include-Dateien        *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Init INC_POOL
function initIncludePool ($poolName) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'poolName=' . $poolName);
	$GLOBALS['inc_pool'][$poolName] = array();
}

// Setter for INC_POOL
function setIncludePool ($poolName, $includePool) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'poolName=' . $poolName);
	$GLOBALS['inc_pool'][$poolName] = (array) $includePool;
}

// Getter for INC_POOL
function getIncludePool ($poolName) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'poolName=' . $poolName);
	// Default is empty pool
	$poolArray = array();

	// Is it set?
	if (isset($GLOBALS['inc_pool'][$poolName])) {
		// Return found pool (array)
		$poolArray = $GLOBALS['inc_pool'][$poolName];
	} // END - if

	// Return it
	return $poolArray;
}

// Count INC_POOL
function countIncludePool ($poolNAme) {
	// Is the pool initialized?
	if (!isset($GLOBALS['inc_pool'][$poolNAme])) {
		// Not initialized with initIncludePool()
		reportBug(__FUNCTION__, __LINE__, sprintf("Pool %s is not initialized.", $poolNAme));
	} // END - if

	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'poolNAme=' . $poolNAme);
	return count($GLOBALS['inc_pool'][$poolNAme]);
}

// Merge INC_POOL into given
function mergeIncludePool ($poolName, $includeArray) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'poolNAme=' . $poolNAme);
	setIncludePool($poolName, merge_array(getIncludePool($poolName), $includeArray));
}

// Add single include file to INC_POOL
function addIncludeToPool ($poolName, $includeFileName) {
	// Is the pool initialized?
	if (!isset($GLOBALS['inc_pool'][$poolName])) {
		// Not initialized with initIncludePool()
		reportBug(__FUNCTION__, __LINE__, sprintf("Pool %s is not initialized, not adding include %s.", $poolName, $includeFileName));
	} // END - if

	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'poolNAme=' . $poolNAme);
	array_push($GLOBALS['inc_pool'][$poolName], $includeFileName);
}

// Remove an include file from INC_POOL
function removeIncludeFromPool ($poolNAme, $inc) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'poolNAme=' . $poolNAme);
	// First look it up
	$key = array_search($inc, getIncludePool($poolNAme));

	// Is it valid?
	if ($key !== FALSE) {
		// Then remove it
		unset($GLOBALS['inc_pool'][$poolNAme][$key]);

		// And sort the list
		asort($GLOBALS['inc_pool'][$poolNAme]);
	} // END - if
}

// Load the whole include pool
function loadIncludePool ($poolName) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'poolNAme=' . $poolNAme.' - START');
	foreach (getIncludePool($poolName) as $includeFileName) {
		//* DEBUG: */ debugOutput(__FUNCTION__.':includeFileName='.$includeFileName);
		loadIncludeOnce($includeFileName);
	} // END - foreach
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'poolNAme=' . $poolNAme.' - END');

	// Remove it
	initIncludePool($poolName);
}

// Loads an include file and logs any missing files for debug purposes
function loadInclude ($includeFileName) {
	// Is there cache?
	if (!isset($GLOBALS['inc_loaded'][$includeFileName])) {
		// Is the include file there?
		if (!isIncludeReadable($includeFileName)) {
			// Not there so log it
			reportBug(__FUNCTION__, __LINE__, sprintf('Include file %s not found or deprecated.', $includeFileName));
		} // END - if

		// Add the path. This is why we need a trailing slash in config.php
		$GLOBALS['inc_loaded'][$includeFileName] = getPath() . $includeFileName;
	} // END - if

	// Try to load it
	include($GLOBALS['inc_loaded'][$includeFileName]);
}

// Loads an include file once
function loadIncludeOnce ($inc) {
	// Is it not loaded?
	if (!isset($GLOBALS['inc_loaded'][$inc])) {
		// Then try to load it
		loadInclude($inc);
	} // END - if
}

// Checks whether an include file (non-FQFN better) is readable
function isIncludeReadable ($inc) {
	// Is there cache?
	if (!isset($GLOBALS['inc_readable'][$inc])) {
		// Construct FQFN
		$fqfn = getPath() . $inc;

		// Is it readable and at least 50 byte large? Include file sizes < 50 byte are always deprecated
		$GLOBALS['inc_readable'][$inc] = ((isFileReadable($fqfn)) && (filesize($fqfn) >= 50));
	} // END - if

	// Return result
	return $GLOBALS['inc_readable'][$inc];
}

// [EOF]
?>
