<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 12/12/2008 *
 * ===================                          Last change: 12/12/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : pool-bonus.php                                   *
 * -------------------------------------------------------------------- *
 * Short description : Sends queued bonus mails from the pool           *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Sendet freigegebene Bonus-Mails aus dem Pool     *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2016 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Don't run on daily reset
if ((isResetModeEnabled()) || (!isHtmlOutputMode())) {
	// Skip here
	return FALSE;
} elseif ((!isExtensionActive('bonus')) || (!isExtensionActive('other'))) {
	// Abort if extension bonus/other are not active
	return FALSE;
}

// Only send bonus mail when bonus extension is active and maximum send-mails is not reached
if ($GLOBALS['pool_cnt'] < getMaxSend()) {
	// Default is ext-html_mail is gone
	$add = '';

	// Send out bonus mails?
	if (isExtensionActive('html_mail')) {
		//  With HTML support
		$add = ', `html_msg`';
	} // END - if

	// Run query
	$result_main = sqlQuery("SELECT
	`id`,
	`subject`,
	`text`,
	`receivers`,
	`points`,
	`time`,
	`data_type`,
	`timestamp`,
	`url`,
	`cat_id`,
	`target_send`
	" . $add . "
FROM
	`{?_MYSQL_PREFIX?}_bonus`
WHERE
	`data_type`='NEW'
ORDER BY
	`timestamp` DESC", __FILE__, __LINE__);

	// Some mails left?
	if (!ifSqlHasZeroNumRows($result_main)) {
		// Init SQLs here
		initSqls();

		// Send these mails away...
		$count2 = '';
		while ($mailData = sqlFetchArray($result_main)) {
			// Message is active in queue
			sqlQueryEscaped("UPDATE `{?_MYSQL_PREFIX?}_bonus` SET `data_type`='QUEUE' WHERE `id`=%s LIMIT 1",
				array(bigintval($mailData['id'])), __FILE__, __LINE__);

			// Default is no users left
			$receivers = array();

			// "Explode" all receivers into an array
			if (isInString(';', $mailData['receivers'])) {
				// There's more than one receiver in the list...
				$receivers = explode(';', $mailData['receivers']);
			} elseif (!empty($mailData['points'])) {
				// Only one user left
				$receivers = array($mailData['receivers']);
			}
			$temporaryReceivers = $receivers;

			/*
			 * Now, if we are good little boys and girls Santa Claus left us
			 * some user-ids and send out mails to them.
			 */
			foreach ($receivers as $key => $userid) {
				// Load personal data
				if (fetchUserData($userid)) {
					// The final receiver does exists so we can continue...
					//* DEBUG: */ debugOutput('OK!/L:'.__LINE__);

					// Remove receiver from list
					$status = removeReceiver($temporaryReceivers, $key, $userid, $mailData['id'], $mailData['id'], TRUE);

					// Did it work?
					switch ($status) {
						case 'done': // Done!
							// Prepare the mail
							$mailText = loadEmailTemplate('member_bonus_pool_normal', $mailData, $userid);

							// Send mail away
							if (isset($mailData['html_msg'])) {
								// Send HTML?
								sendEmail(getUserData('userid'), $mailData['subject'], $mailText, $mailData['html_msg']);
							} else {
								// No HTML mail!
								sendEmail(getUserData('userid'), $mailData['subject'], $mailText);
							}

							// Count one up and remove entry from dummy array
							$GLOBALS['pool_cnt']++; unset($temporaryReceivers[$key]);

							// Is ext-user installed?
							if (isExtensionInstalledAndNewer('user', '0.1.4')) {
								// Debug message
								//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'userid=' . $userid . ',mailData[id]=' . $mailData['id']);
								// Update mails received for receiver
								addSql(sprintf("UPDATE `{?_MYSQL_PREFIX?}_user_data` SET `emails_received`=`emails_received`+1 WHERE `userid`=%s LIMIT 1", bigintval($userid)));
							} // END - if

							// Is the size of pool reached or nothing to sent left?
							if (($GLOBALS['pool_cnt'] >= getMaxSend()) || (countSelection($temporaryReceivers) == 0)) {
								// Yes, it is. So abort here
								break;
							} // END - if
							break;

						default: // Unknown return type
							logDebugMessage(__FILE__, __LINE__, 'Unknown status ' . $status . ' detected. pool_id=' . $mailData['id'] . ',userid=' . $mailData['userid'] . ',stats_id=' . $stats_id);
							break;
					} // END - switch
				} // END - if
			} // END - foreach

			// Run all SQLs
			runFilterChain('run_sqls');

			// Update mediadata if version is 0.0.4 or higher
			if (isExtensionInstalledAndNewer('mediadata', '0.0.4')) {
				// Update entry (or add missing
				$P = $GLOBALS['pool_cnt'];
				if (!empty($count2) && empty($GLOBALS['pool_cnt'])) $P = $count2;
				//* DEBUG: */ debugOutput('+MEDIA/L:'.__LINE__.'/'.$P.'+');
				updateMediadataEntry(array('total_send', 'bonus_send'), 'add', $P);
			} // END - if

			// Close sending system
			//* DEBUG: */ debugOutput('-L:'.__LINE__.'/'.countSelection($temporaryReceivers).'-');
			if (countSelection($temporaryReceivers) == 0) {
				// Queue reached!
				sqlQueryEscaped("UPDATE
	`{?_MYSQL_PREFIX?}_bonus`
SET
	`data_type`='SEND',
	`target_send`=0,
	`receivers`=''
WHERE
	`id`=%s
LIMIT 1",
					array(bigintval($mailData['id'])), __FILE__, __LINE__);
				//* DEBUG: */ debugOutput('*L:'.__LINE__.'*');

				// Update mediadata if version is 0.0.4 or higher
				if (isExtensionInstalledAndNewer('mediadata', '0.0.4')) {
					// Update entry (or add missing)
					//* DEBUG: */ debugOutput('*MEDIA/L:'.__LINE__.'*');
					updateMediadataEntry(array('total_orders', 'bonus_orders'), 'add', 1);
				} // END - if
			} elseif ($GLOBALS['pool_cnt'] >= getMaxSend()) {
				// Update bonus pool
				sqlQueryEscaped("UPDATE
	`{?_MYSQL_PREFIX?}_bonus`
SET
	`data_type`='NEW',
	`target_send`=%s,
	`receivers`='%s'
WHERE
	`id`=%s
LIMIT 1",
					array(
						countSelection($temporaryReceivers),
						implode(';', $temporaryReceivers),
						bigintval($mailData['id'])
					), __FILE__, __LINE__);
				//* DEBUG: */ debugOutput('*L:'.__LINE__.'<pre>'.print_r($temporaryReceivers, TRUE).'</pre>!!!');

				// Run any SQLs
				runFilterChain('run_sqls');
				break;
			}
		} // END - while
	} // END - if

	// Free memory
	sqlFreeResult($result_main);

	// Remove variable
	unset($mailText);
} // END - if

// [EOF]
?>
